@@ -60,10 +60,6 @@ Actor::~Actor () {
     //m_agent->medias ().remove (this);
 }
 
-Mrl *Actor::mrl () {
-    return m_node ? m_node->mrl () : NULL;
-}
-
 KDE_NO_EXPORT void Actor::dismiss () {
     delete this;
 }
@@ -508,10 +504,10 @@ AudioVideoActor::~AudioVideoActor () {
 
 bool AudioVideoActor::play () {
     Process *process = m_agent->control ()->process ();
-    if (process && process->playing () && process->mrl () == mrl ()) {
-        errorLog () << "play: already playing " << mrl ()->src << endl;
-    } else if (m_agent->control ()->requestPlayURL (mrl ())) {
-        if (!mrl ()->audio_only)
+    if (process && process->playing () && process->mrl () == sourceMrl ()) {
+        errorLog () << "play: already playing " << sourceMrl ()->src << endl;
+    } else if (m_agent->control ()->requestPlayURL (sourceMrl ())) {
+        if (!sourceMrl ()->audio_only)
             m_agent->control ()->viewArea ()->setAudioVideoNode (m_node);
         m_node->defer ();
         return true;
@@ -521,8 +517,10 @@ bool AudioVideoActor::play () {
 
 void AudioVideoActor::stop () {
     Process *process = m_agent->control ()->process ();
-    if (process)
+    if (process && process->mrlBackEndNotify () == this) {
         process->stop ();
+        process->setMrlBackEndNotify (NULL);
+    }
     m_agent->control ()->viewArea ()->setAudioVideoNode (NULL);
 }
 
@@ -535,7 +533,54 @@ void AudioVideoActor::pause () {
 void AudioVideoActor::unpause () {
     Process *process = m_agent->control ()->process ();
     if (process && process->playing ())
-        process->play (NULL);
+        process->play (String ());
+}
+
+Mrl *AudioVideoActor::sourceMrl ()
+{
+    return m_node ? m_node->mrl () : NULL;
+}
+
+void AudioVideoActor::referenceMrl (const String &uri)
+{
+    Mrl *_mrl = sourceMrl ();
+    if (_mrl && !uri.isEmpty ()) {
+        for (Node *n = _mrl; n; n = n->parentNode ()) {
+            Mrl *m = n->mrl ();
+            if (m && m->src == uri)
+                return;
+        }
+        for (Node *n = _mrl->firstChild (); n; n = n->nextSibling ()) {
+            Mrl *m = n->mrl ();
+            if (m && m->src == uri)
+                return;
+        }
+
+        NodePtr doc = _mrl->document ();
+        Node *n = new GenericURL (doc, uri, String ());
+        n->id = id_node_ref_url;
+        n->setAuxiliaryNode (true);
+        _mrl->appendChild (n);
+
+        m_agent->control ()->updatePlaylistView ();
+    }
+}
+
+void AudioVideoActor::infoMessage (const String &msg) {
+    if (sourceMrl ())
+        sourceMrl ()->document ()->message (MsgInfoString, (void *) &msg);
+}
+
+void AudioVideoActor::readyToPlay (Process *process)
+{
+    Mrl *mrl = sourceMrl ();
+    if (mrl) {
+        process->setMrlBackEndNotify (this);
+        process->setRepeat (mrl->repeat);
+        process->setHasVideo (!mrl->audio_only);
+        // TODO set aspect/crop/http_object args
+        process->play (mrl->absolutePath ());
+    }
 }
 
 //------------------------%<----------------------------------------------------
