@@ -322,7 +322,7 @@ static void callback_file_go (GtkButton *button, Application *app) {
 
 static void callback_file_new (GtkButton * button, Application * app) {
     newFile (app);
-    gtk_window_set_title (app->window, "");
+    gtk_window_set_title (app->window, "kmplayer");
 }
 
 static gboolean
