@@ -193,26 +193,80 @@ bool parseTime (const String & val, int & dur /*, const String & dateformat*/);
 
 class KMPLAYER_NO_EXPORT SmilTextProperties {
 public:
+    enum Align { AlignInherit, AlignLeft, AlignCenter, AlignRight };
+    enum FontWeight { WeightNormal, WeightBold, WeightInherit };
+    enum Spacing { SpaceDefault, SpacePreserve };
+    enum Style {
+        StyleNormal, StyleItalic, StyleOblique, StyleRevOblique, StyleInherit
+    };
+    enum TextDirection { DirLtr, DirRtl, DirLtro, DirRtlo, DirInherit };
+    enum TextMode { ModeAppend, ModeReplace, ModeInherit };
+    enum TextPlace { PlaceStart, PlaceCenter, PlaceEnd, PlaceInherit };
+    enum TextWrap { Wrap, NoWrap, WrapInherit };
+    enum TextWriting { WritingLrTb, WritingRlTb, WritingTbLr, WritingTbRl };
+
     void init ();
     bool parseParam (const TrieString &name, const String &value);
     void mask (const SmilTextProperties &props);
 
     String font_family;
+    String text_style;
     int font_color;
     int background_color;
-    enum { DirLtr, DirRtl, DirLtro, DirRtlo, DirInherit } text_direction;
-    int font_size;
-    enum Style {
-        StyleNormal, StyleItalic, StyleOblique, StyleRevOblique, StyleInherit
-    } font_style;
-    enum { WeightNormal, WeightBold, WeightInherit } font_weight;
-    enum { ModeAppend, ModeReplace, ModeInherit } text_mode;
-    enum { PlaceStart, PlaceCenter, PlaceEnd, PlaceInherit } text_place;
-    String text_style;
-    enum { Wrap, NoWrap, WrapInherit } text_wrap;
-    enum { SpaceDefault, SpacePreserve } space;
-    enum { WritingLrTb, WritingRlTb, WritingTbLr, WritingTbRl } text_writing;
-    enum Align { AlignInherit, AlignLeft, AlignCenter, AlignRight } text_align;
+    unsigned char text_direction;
+    unsigned char font_style;
+    unsigned char font_weight;
+    unsigned char text_mode;
+    unsigned char text_place;
+    unsigned char text_wrap;
+    unsigned char space;
+    unsigned char text_writing;
+    unsigned char text_align;
+    unsigned char padding;
+    short font_size;
+};
+
+class KMPLAYER_NO_EXPORT SmilColorProperty {
+public:
+    void init ();
+    void setColor (const String &value);
+    void setOpacity (const String &value);
+
+    unsigned int color;
+    int opacity;
+};
+
+class KMPLAYER_NO_EXPORT MediaOpacity {
+public:
+    void init ();
+
+    unsigned short opacity;
+    unsigned short bg_opacity;
+};
+
+class KMPLAYER_NO_EXPORT TransitionModule {
+public:
+    TransitionModule ()
+     : trans_start_time (0),
+       trans_out_timer (NULL),
+       trans_out_active (false) {}
+
+    void init ();
+    void begin (Node *n, Runtime *r);
+    bool handleMessage (Node *n, Runtime *r, Surface *s, MessageType m, void *);
+    void cancelTimer (Node *n);
+    void finish (Node *n);
+
+    NodePtrW trans_in;
+    NodePtrW trans_out;
+    NodePtrW active_trans;
+    unsigned int trans_start_time;
+    unsigned int trans_end_time;
+    Command *trans_out_timer;
+    float trans_gain;
+    ConnectionList m_TransformedIn;        // transIn ready
+    ConnectionLink transition_updater;
+    bool trans_out_active;
 };
 
 //-----------------------------------------------------------------------------
@@ -232,7 +286,6 @@ const short id_node_par = 111;
 const short id_node_seq = 112;
 const short id_node_switch = 113;
 const short id_node_excl = 114;
-const short id_node_img = 120;
 const short id_node_text = 122;
 const short id_node_ref = 123;
 const short id_node_brush = 124;
@@ -252,13 +305,14 @@ const short id_node_priorityclass = 143;
 const short id_node_div = 144;
 const short id_node_span = 145;
 const short id_node_p = 146;
+const short id_node_br = 147;
 const short id_node_anchor = 150;
 const short id_node_area = 151;
 const short id_node_state_data = 151;
 const short id_node_first = id_node_smil;
 const short id_node_first_timed_mrl = id_node_body;
 const short id_node_last_timed_mrl = id_node_animate_motion;
-const short id_node_first_mediatype = id_node_img;
+const short id_node_first_mediatype = id_node_text;
 const short id_node_last_mediatype = id_node_brush;
 const short id_node_first_group = id_node_body;
 const short id_node_last_group = id_node_excl;
@@ -367,14 +421,15 @@ public:
     CalculatedSizer sizes;
 
     int z_order;
-    unsigned int background_color;
-    int bg_opacity;
+    SmilColorProperty background_color;
+    MediaOpacity media_opacity;
     String background_image;
     enum BackgroundRepeat {
         BgRepeat, BgRepeatX, BgRepeatY, BgNoRepeat, BgInherit
     } bg_repeat;
     ShowBackground show_background;
     Fit fit;
+    SmilTextProperties font_props;
     ConnectionList m_AttachedMediaTypes;   // active attached mediatypes
 protected:
     RegionBase (NodePtr & d, short id);
@@ -672,24 +727,16 @@ public:
     Runtime *runtime;
     SurfacePtrW sub_surface;
     NodePtrW external_tree; // if src points to playlist, the resolved top node
-    NodePtrW trans_in;
-    NodePtrW trans_out;
-    NodePtrW active_trans;
+    TransitionModule transition;
     NodePtrW region_node;
     String m_type;
     CalculatedSizer sizes;
     CalculatedSizer *pan_zoom;
     Fit fit;
-    unsigned int background_color;
-    int opacity;
-    int bg_opacity;
+    SmilColorProperty background_color;
+    MediaOpacity media_opacity;
     unsigned int bitrate;
-    unsigned int trans_start_time;
-    unsigned int trans_end_time;
-    float trans_gain;
-    Command *trans_out_timer;
     enum { sens_opaque, sens_transparent, sens_percentage } sensitivity;
-    bool trans_out_active;
 
 protected:
     virtual void prefetch ();
@@ -698,10 +745,8 @@ protected:
 
     MouseListeners mouse_listeners;
     ConnectionList m_MediaAttached;
-    ConnectionList m_TransformedIn;        // transIn ready
     ConnectionLink region_attach;          // attached to region
     ConnectionLink document_postponed;     // pause audio/video accordantly
-    ConnectionLink transition_updater;
     PostponePtr postpone_lock;
 };
 
@@ -709,25 +754,17 @@ class KMPLAYER_NO_EXPORT RefMediaType : public MediaType {
 public:
     RefMediaType (NodePtr &doc, const String &tag);
     Node *childFromTag (const String & tag);
+    virtual void activate ();
     virtual void begin ();
     virtual void finish ();
+    virtual PlayType playType ();
     virtual void accept (Visitor *);
+    virtual void message (MessageType msg, void *content=NULL);
     virtual void *role (RoleType msg, void *content=NULL);
     virtual void prefetch ();
     virtual void clipStart ();
 };
 
-class KMPLAYER_NO_EXPORT ImageMediaType : public MediaType {
-public:
-    ImageMediaType (NodePtr & d);
-    Node *childFromTag (const String & tag);
-    PlayType playType () { return play_type_image; }
-    virtual void activate ();
-    virtual void accept (Visitor *);
-    virtual void message (MessageType msg, void *content=NULL);
-    virtual void prefetch ();
-};
-
 class KMPLAYER_NO_EXPORT TextMediaType : public MediaType {
 public:
     TextMediaType (NodePtr & d);
@@ -770,6 +807,9 @@ public:
 
     Surface *surface ();
 
+    SmilColorProperty background_color;
+    MediaOpacity media_opacity;
+    TransitionModule transition;
     SmilTextProperties props;
     SurfacePtrW text_surface;
     NodePtrW region_node;
