@@ -147,6 +147,10 @@ void SourceDocument::message (MessageType msg, void *data) {
                 data ? (const char *) *((String *) data) : "", false);
         return;
 
+    case MsgBannerString:
+        m_control->errorMsg (*((String *) data));
+        return;
+
     case MsgAccessKey:
         for (Connection *c = m_KeyListeners.first(); c; c = m_KeyListeners.next ())
             if (c->payload && c->connecter) {
@@ -205,7 +209,8 @@ void *SourceDocument::role (RoleType msg, void *data) {
 
 
 Control::Control (Application * a)
-    : m_process (0L), m_doc_timer (0), progress_banner (NULL), mplayer (NULL),
+    : m_process (0L), m_doc_timer (0), progress_banner (NULL),
+      mplayer (NULL), gstreamer_mplayer (NULL),
       playlist_data (0L), m_config (new Config), m_app (a),
       paint_timer (0), update_tree_timer (0), blanking_timer (0),
       in_progress_update (false),
@@ -249,19 +254,10 @@ void Control::init () {
 
     use_xvideo = true;
    //media_server = new OssoMediaServer (this, m_app->osso_context, use_xvideo);
-    MPlayer *gst = new MPlayer (this, "osso-media-server");
-    gst->setPlayer ("/opt/kmplayer/bin/kgstplayer");
-    gst->setAudioDriver ("pulse");
-    gst->setVideoDriver ("xv");
-    gst->setAutoSubTitles (true);
-    gst->setSupportsPanzoom (true);
-    media_server = gst;
+    media_server = GstreamerMPlayerProcess::createGstreamerPlayer (this);
     if (glob ("/usr/bin/mplayer", 0, NULL, &globbuf) != GLOB_NOMATCH) {
-        MPlayer *mp = new MPlayer (this, "mplayer");
-        mp->setAudioDriver ("pulse");
-        mp->setVideoDriver ("xv:ck-method=auto");
-        //mp->setAudioDriver ("pulse");
-        mplayer = mp;
+        mplayer = GstreamerMPlayerProcess::createMPlayer (this);
+        gstreamer_mplayer = new GstreamerMPlayerProcess (this);
     }
     globfree (&globbuf);
 #ifdef __ARMEL__
@@ -476,7 +472,16 @@ static PlayListData *playListData (PlayListData *pld, short id)
 }
 
 void Control::stop (bool keep_fullscreen) {
+    bool was_document = false;
+    String url;
+    Mrl *mrl = m_process->mrl ();
+    if (mrl) {
+        was_document = mrl->id == id_node_document;
+        url = mrl->src;
+    }
+
     back_request = 0L;
+
     if (playlist_data->document &&
             playlist_data->document->state > Node::state_init) {
         playlist_data->document->reset ();
@@ -492,15 +497,14 @@ void Control::stop (bool keep_fullscreen) {
     }
 
     Node *recent = recents->firstChild ();
-    Mrl *mrl = m_process->mrl ();
-    if (recent && mrl &&
+    if (recent &&
             m_process->isLocalFile () &&
-            recent->mrl ()->src == mrl->src) {
+            recent->mrl ()->src == url) {
         Mrl *recent_mrl = recent->mrl ();
         String ts = recent_mrl->getAttribute ("clipBegin");
         int pos = m_process->userStopPosition ();
         pos = pos > 50 ? pos - 50 : 0;
-        if (mrl->id == id_node_document)
+        if (was_document)
             mrl->clip_begin = 10 * pos;
         String nts;
         int ds = pos % 10;
@@ -607,11 +611,14 @@ void Control::playCurrent () {
                 return; // FIXME
         }
         back_request = 0L;
-        if (!mrl->src.isEmpty () && mrl->resolved) {
+        if (!mrl->src.isEmpty () && mrl->resolved &&
+                mrl->media_info &&
+                (ActorAgent::Audio == mrl->media_info->type ||
+                 ActorAgent::AudioVideo == mrl->media_info->type)) {
             updateViewAreaSizes ();
             if (m_process->hasXv ())
                 XSync (GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (VIEWAREA(m_app)->viewWidget ())), false);
-            m_process->play (mrl);
+            static_cast<AudioVideoActor *> (mrl->media_info->media)->readyToPlay (m_process);
         } else
             errorLog() << "playCurrent and mrl not resolved" << endl;
     }
@@ -682,6 +689,8 @@ bool Control::setProcess (const char *pname) {
     Process *old_process = m_process;
     if (mplayer && !strcmp (pname, mplayer->name ())) {
         m_process = mplayer;
+    } else if (gstreamer_mplayer && !strcmp(pname, gstreamer_mplayer->name())) {
+        m_process = gstreamer_mplayer;
 #ifdef __ARMEL__
     } else if (!strcmp (pname, npp->name ())) {
         m_process = npp;
@@ -719,7 +728,8 @@ bool Control::setProcess (Mrl *mrl) {
     if (pname.isEmpty () && !mrl->mimetype.isEmpty ())
         pname = processForMimetype (this, mrl->mimetype);
     if (pname.isEmpty ())
-        pname = m_config->readEntry (GCONF_KEY_PLAYER, "osso-media-server");
+        pname = m_config->readEntry (GCONF_KEY_PLAYER,
+                gstreamer_mplayer ? "gstreamer or mplayer":"osso-media-server");
     if (!m_process || pname != m_process->name ()) {
         setProcess ((const char *) pname);
         changed = true;
@@ -1383,14 +1393,14 @@ gboolean cb_keyPressed (GtkWidget *, GdkEventKey * e, Application *app) {
                 app_normal_screen (app);
             }
             break;
-        case GDK_F7: //HILDON_HARDKEY_INCREASE
+        /*case GDK_F7: //HILDON_HARDKEY_INCREASE
             if (CONTROL(app)->process ()->playing ())
                 CONTROL(app)->process ()->volume (2, false);
             break;
         case GDK_F8: //HILDON_HARDKEY_DECREASE
             if (CONTROL(app)->process ()->playing ())
                 CONTROL(app)->process ()->volume (-2, false);
-            break;
+            break;*/
         case GDK_Left:
             if (gtk_window_get_focus (app->window) == app->playlist_view ||
                     testFullscreenPlaylist (app))
@@ -1541,14 +1551,18 @@ void playerMenuActivate (GtkMenuItem *item, Application *app) {
     if (ctrl->mplayer) {
         hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
                 ctrl->mplayer->name ());
+        hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
+                ctrl->gstreamer_mplayer->name ());
         if (player == ctrl->mplayer->name ())
             idx = 1;
+        else if (player == ctrl->gstreamer_mplayer->name ())
+            idx = 2;
     }
 #ifdef __ARMEL__
     hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector),
             ctrl->npp->name ());
     if (player == ctrl->npp->name ())
-        idx = ctrl->mplayer ? 2 : 1;
+        idx = ctrl->mplayer ? 3 : 1;
 #endif
     hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, idx);
     gtk_window_set_title (GTK_WINDOW (dialog), "Select Player");
@@ -1559,6 +1573,9 @@ void playerMenuActivate (GtkMenuItem *item, Application *app) {
         Process *p = ctrl->media_server;
         if (ctrl->mplayer && ctrl->selected_player == ctrl->mplayer->name ())
             p = ctrl->mplayer;
+        else if (ctrl->gstreamer_mplayer &&
+                ctrl->selected_player == ctrl->gstreamer_mplayer->name ())
+            p = ctrl->gstreamer_mplayer;
 #ifdef __ARMEL__
         else if (ctrl->selected_player == ctrl->npp->name ())
             p = ctrl->npp;
@@ -1788,6 +1805,30 @@ void playlistMenuActivate (GtkMenuItem *, Application *app) {
 
 namespace {
 
+class Download : public Element, public IOJobListener
+{
+public:
+    Download (NodePtr &doc, Control *n, int fd = -1);
+    ~Download ();
+
+    virtual void closed();
+    virtual void begin();
+    virtual const char *nodeName () const { return "download"; }
+
+    virtual void jobData (IOJob *job, ByteArray & data);
+    virtual void jobResult (IOJob *job);
+    virtual void redirected (IOJob * job, const String & uri) {}
+
+    String title;
+    String file;
+    String uri;
+    IOJob *job;
+    Control *notify;
+    off_t size;
+    off_t bytes;
+    int fd;
+};
+
 class DownloadDocument : public FileDocument
 {
 public:
@@ -1813,6 +1854,92 @@ public:
 
 }
 
+Download::Download (NodePtr &d, Control *n, int _fd)
+    : Element (d, id_node_download),
+      job (NULL), notify (n),
+      size (0), bytes (0), fd (_fd)
+{}
+
+Download::~Download ()
+{
+    if (job) {
+        job->kill ();
+        ::close (fd);
+    }
+}
+
+void Download::closed ()
+{
+    file = getAttribute ("file");
+    uri = getAttribute (Ids::attr_src);
+    title = getAttribute (Ids::attr_title);
+    Element::closed ();
+}
+
+void Download::begin ()
+{
+    setState (state_began);
+    bytes = 0;
+    size = 0;
+    debugLog() << "Download " << uri << " as " << file << " cap:" << title << endl;
+    job = asyncGet (this, uri);
+    if (fd > -1) {
+        job->start ();
+    } else {
+        String sz = getAttribute ("size");
+        if (sz.isEmpty ()) {
+            fd = ::open (file, O_WRONLY | O_CREAT | O_TRUNC, 0664);
+        } else {
+            size = strtoll (sz, NULL, 10);
+            fd = ::open (file, O_WRONLY | O_APPEND, 0664);
+            bytes = ::lseek (fd, 0, SEEK_END);
+        }
+        if (fd > 0)
+            job->start (bytes);
+    }
+    notify->updateDownload (this, 0);
+}
+
+void Download::jobData (IOJob *jb, ByteArray &data)
+{
+    int old_perc = size > 0 ? int (100.0 * bytes / size) : 0;
+    if (!bytes) {
+        size = jb->contentLength ();
+        if (size > 0)
+            setAttribute ("size", String::number (size));
+    }
+    bytes += data.size ();
+    int new_perc = size > 0 ? int (100.0 * bytes / size) : 0;
+    if (new_perc >= 100)
+        new_perc = 99;
+    if (old_perc != new_perc)
+        notify->updateDownload (this, new_perc);
+    if (data.size ()) {
+        int nr = write (fd, data.data (), data.size ());
+        if (nr < data.size ()) {
+            String err ("Couldn't write data");
+            if (nr < 0)
+                err += Char (' ') + String (strerror (errno));
+            jb->kill (false);
+            document ()->message (MsgBannerString, &err);
+        }
+    }
+}
+
+void Download::jobResult (IOJob *jb)
+{
+    if (fd > 0)
+        close (fd);
+    fd = -1;
+    job = NULL;
+    if (!jb->error ()) {
+        notify->updateDownload (this, 100);
+        NodePtr safe (this);
+        parentNode ()->removeChild (this); // deletes this
+        notify->updateDownloads ();
+    }
+}
+
 Node *Control::downloads ()
 {
     if (!downloads_doc)
@@ -1868,6 +1995,85 @@ void Control::updateDownload (Node *node, int percentage)
     }
 }
 
+static String genFileName (const String &str, bool skip_on_invalid) {
+    String name;
+    bool valid = true;
+    for (const char *s = (const char *)str; s && *s; ++s)
+        switch (*s) {
+            case '\n':
+            case ';':
+            case ':':
+            case '*':
+            case '?':
+            case ',':
+            case '&':
+            case '>':
+            case '<':
+            case '|':
+            case '"':
+            case '\'':
+            case '/':
+                if (!skip_on_invalid)
+                    return String ();
+                break;
+            default:
+                name += Char (*s);
+        }
+    return name;
+}
+
+void Control::createDownload (const String &url, const String &target_title) {
+    String name, err;
+    String title = target_title;
+    URL uri (url);
+
+    if (uri.isLocalFile ()) {
+        err = "Refuse to download a local file";
+    } else if (uri.protocol () != "http" &&
+            uri.protocol () != "https" &&
+            uri.protocol () != "ftp" &&
+            uri.protocol () != "sftp") {
+        err = String ("Cannot download from protocol ") + uri.protocol ();
+    } else {
+        int pos = url.lastIndexOf (Char ('/'));
+        if (pos > 0)
+            name = genFileName ((const char *) url.mid (pos + 1), false);
+        if (name.isEmpty ())
+            name = title;
+        if (!name.isEmpty ()) {
+            if (title.isEmpty ())
+                title = name;
+            name = genFileName ((const char *) name, true);
+        }
+        String path = String (m_app->save_folder);
+        if (!name.isEmpty ())
+            path += name;
+        gchar *filename = app_file_chooser (m_app, (const char *) path,
+                GTK_FILE_CHOOSER_ACTION_SAVE);
+        if (filename) {
+            int fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
+            if (fd > 0) {
+                NodePtr doc = downloads ();
+                Element *n = new Download (doc, this, fd);
+                doc->appendChild (n);
+                n->setAttribute (Ids::attr_src, url);
+                n->setAttribute (Ids::attr_title, title);
+                n->setAttribute ("file", filename);
+                n->closed ();
+                n->begin ();
+                updateDownloads ();
+            } else {
+                err = "Couldn't open file ";
+                err += String (filename) + Char (' ');
+                err += strerror (errno);
+            }
+            app_store_save_directory (m_app, filename);
+        }
+    }
+    if (!err.isEmpty ())
+        errorMsg (err);
+}
+
 void Control::videoDetected ()
 {
     Mrl *mrl = m_process ? m_process->mrl () : NULL;
@@ -1880,30 +2086,6 @@ void Control::videoDetected ()
     app_update_screen_blanking (m_app);
 }
 
-void Control::insertMrl (Mrl *mrl, const String &uri)
-{
-    if (mrl && !uri.isEmpty ()) {
-        for (Node *n = mrl; n; n = n->parentNode ()) {
-            Mrl *m = n->mrl ();
-            if (m && m->src == uri)
-                return;
-        }
-        for (Node *n = mrl->firstChild (); n; n = n->nextSibling ()) {
-            Mrl *m = n->mrl ();
-            if (m && m->src == uri)
-                return;
-        }
-
-        NodePtr doc = mrl->document ();
-        Node *n = new GenericURL (doc, uri, String ());
-        n->id = id_node_ref_url;
-        n->setAuxiliaryNode (true);
-        mrl->appendChild (n);
-
-        updateTree (m_app, 0);
-    }
-}
-
 static gboolean downloadPageDestroyed (GtkObject *, Application *app)
 {
     debugLog() << "Download page destroyed\n";
@@ -2019,7 +2201,7 @@ static void addPlainDownload (GtkButton *, Application *app)
             "Please enter the link to download", "http://");
     if (!url.isEmpty () && url != "http://") {
         Control *ctrl = CONTROL(app);
-        ctrl->downloader->createDownload (url, String ());
+        ctrl->createDownload (url, String ());
     }
 }
 
