@@ -38,6 +38,7 @@ class Mrl;
 class ElementPrivate;
 class Visitor;
 class MediaInfo;
+class Process;
 
 /*
  * Base class for objects that will be used as SharedPtr/WeakPtr pointers.
@@ -245,7 +246,8 @@ enum MessageType
     MsgChildFinished,
     MsgShowMeta,
 
-    MsgInfoString                // String*
+    MsgInfoString,               // String*
+    MsgBannerString              // String*
 };
 
 enum RoleType
@@ -640,6 +642,15 @@ public:
     bool has_played;
 };
 
+class KMPLAYER_NO_EXPORT MrlBackEndNotify
+{
+public:
+    virtual Mrl *sourceMrl () = 0;
+    virtual void referenceMrl (const String &url) = 0;
+    virtual void infoMessage (const String &msg) = 0;
+    virtual void readyToPlay (Process *process) = 0;
+};
+
 /**
  * Document listener interface
  */
