@@ -40,9 +40,10 @@ using namespace KMPlayer;
 
 
 Process::Process (ProcessNotify * ctrl, const char * n, bool xv)
- : m_notify (ctrl), m_viewer (0L), m_name (n),
+ : m_notify (ctrl), m_mrl_notify (NULL), m_viewer (0L), m_name (n),
       m_state (NotRunning), m_old_state (NotRunning),
-      start_timer (0), no_video (false), has_xv (xv), is_local_file (false)
+      start_timer (0), m_repeat (0),
+      no_video (false), has_xv (xv), is_local_file (false)
       {}
 
 Process::~Process () {
@@ -76,25 +77,25 @@ KMPLAYER_NO_EXPORT bool Process::seek (int /*pos*/, bool /*absolute*/) {
     return false;
 }
 
-KMPLAYER_NO_EXPORT bool Process::volume (int /*pos*/, bool /*absolute*/) {
-    return false;
-}
+//KMPLAYER_NO_EXPORT bool Process::volume (int /*pos*/, bool /*absolute*/) {
+    //return false;
+//}
 
-KMPLAYER_NO_EXPORT bool Process::saturation (int /*pos*/, bool /*absolute*/) {
-    return false;
-}
+//KMPLAYER_NO_EXPORT bool Process::saturation (int /*pos*/, bool /*absolute*/) {
+    //return false;
+//}
 
-KMPLAYER_NO_EXPORT bool Process::hue (int /*pos*/, bool /*absolute*/) {
-    return false;
-}
+//KMPLAYER_NO_EXPORT bool Process::hue (int /*pos*/, bool /*absolute*/) {
+    //return false;
+//}
 
-KMPLAYER_NO_EXPORT bool Process::contrast (int /*pos*/, bool /*absolute*/) {
-    return false;
-}
+//KMPLAYER_NO_EXPORT bool Process::contrast (int /*pos*/, bool /*absolute*/) {
+    //return false;
+//}
 
-KMPLAYER_NO_EXPORT bool Process::brightness (int /*pos*/, bool /*absolute*/) {
-    return false;
-}
+//KMPLAYER_NO_EXPORT bool Process::brightness (int /*pos*/, bool /*absolute*/) {
+    //return false;
+//}
 
 void Process::toggleScale ()
 {}
@@ -159,11 +160,8 @@ KMPLAYER_NO_EXPORT void Process::scheduledStateChanged () {
     start_timer = 0;
 }
 
-KMPLAYER_NO_EXPORT bool Process::play (NodePtr _mrl) {
-    m_mrl = _mrl;
-    Mrl *m = mrl ();
-    m_url = m ? m->absolutePath () : String ();
-    no_video = m ? m->audio_only : false;
+KMPLAYER_NO_EXPORT bool Process::play (const String &url) {
+    m_url = url;
     m_position = m_length = 0;
     m_user_stop_position = 0;
     m_width = m_height = 0;
@@ -172,6 +170,13 @@ KMPLAYER_NO_EXPORT bool Process::play (NodePtr _mrl) {
     return false;
 }
 
+bool Process::setMrlBackEndNotify (MrlBackEndNotify *mrl_notify)
+{
+    m_mrl_notify = mrl_notify;
+    m_repeat = 0;
+    no_video = false;
+}
+
 KMPLAYER_NO_EXPORT bool Process::ready (unsigned int viewer) {
     m_viewer = viewer;
     setState (Ready);
@@ -182,11 +187,6 @@ KMPLAYER_NO_EXPORT unsigned int Process::viewer () const {
     return m_viewer;
 }
 
-KMPLAYER_NO_EXPORT void Process::infoMessage (const String &msg) {
-    if (m_mrl)
-        m_mrl->document ()->message (MsgInfoString, (void *) &msg);
-}
-
 void Process::setUserStop () {
     if (m_state > Buffering && m_length > 0 && is_local_file)
         m_user_stop_position = m_position;
@@ -394,9 +394,9 @@ MPlayer::MPlayer (ProcessNotify *ctrl, const char *nm)
 MPlayer::~MPlayer () {
 }
 
-KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
+KMPLAYER_NO_EXPORT bool MPlayer::play (const String &uri) {
     if (!needs_restarted && playing ()) {
-        if (node ||
+        if (m_mrl_notify ||
                 (m_transition_state == Paused ||
                  (Paused == m_state && m_transition_state != Playing))) {
             m_transition_state = Playing;
@@ -411,7 +411,7 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     if (!needs_restarted)
         panzoom_enabled = false;
     setState (Buffering);
-    Process::play (node);
+    Process::play (uri);
     if (m_url.isEmpty ()) {
         setState (Ready);
         return false;
@@ -471,8 +471,8 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     snprintf (buf, sizeof(buf), " -wid %u ", m_viewer);
     cmdline += String (buf);
     Mrl *mrl = this->mrl ();
-    if (mrl->repeat > 0)
-        cmdline += String (" -loop ") + String::number (mrl->repeat);
+    if (m_repeat > 0)
+        cmdline += String (" -loop ") + String::number (m_repeat);
     if (is_local_file && mrl->clip_begin > 100)
         cmdline += String (" -ss ") + String::number ((int) mrl->clip_begin/100);
     cmdline += Char (' ');
@@ -541,7 +541,6 @@ KMPLAYER_NO_EXPORT bool MPlayer::play (NodePtr node) {
     cmdline += String (quoted);
     g_free (quoted);
     if (unix_process.start (wd, cmdline)) {
-        m_mrl = node;
         setState (Process::Buffering); //FIXME
         return true;
     }
@@ -612,25 +611,25 @@ KMPLAYER_NO_EXPORT bool MPlayer::seek (int pos, bool absolute) {
     return sendCommand (String (buf));
 }
 
-KMPLAYER_NO_EXPORT bool MPlayer::volume (int pos, bool /*absolute*/) {
-    return sendCommand (String ("volume ") + String::number (pos));
-}
+//KMPLAYER_NO_EXPORT bool MPlayer::volume (int pos, bool /*absolute*/) {
+    //return sendCommand (String ("volume ") + String::number (pos));
+//}
 
-KMPLAYER_NO_EXPORT bool MPlayer::saturation (int pos, bool absolute) {
-    return true;
-}
+//KMPLAYER_NO_EXPORT bool MPlayer::saturation (int pos, bool absolute) {
+    //return true;
+//}
 
-KMPLAYER_NO_EXPORT bool MPlayer::hue (int pos, bool absolute) {
-    return true;
-}
+//KMPLAYER_NO_EXPORT bool MPlayer::hue (int pos, bool absolute) {
+    //return true;
+//}
 
-KMPLAYER_NO_EXPORT bool MPlayer::contrast (int pos, bool absolute) {
-    return true;
-}
+//KMPLAYER_NO_EXPORT bool MPlayer::contrast (int pos, bool absolute) {
+    //return true;
+//}
 
-KMPLAYER_NO_EXPORT bool MPlayer::brightness (int pos, bool absolute) {
-    return true;
-}
+//KMPLAYER_NO_EXPORT bool MPlayer::brightness (int pos, bool absolute) {
+    //return true;
+//}
 
 bool MPlayer::calculateScale (int &x, int &y, int &w, int &h)
 {
@@ -683,7 +682,7 @@ KMPLAYER_NO_EXPORT void MPlayer::processExited (UnixProcess *) {
     commands.clear ();
     if (needs_restarted) {
         int pos = m_position;
-        play (m_mrl);
+        play (m_url);
         seek (pos, true);
         if (Paused == m_state)
             pause ();
@@ -809,8 +808,8 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
                             if (m_aspect > 0.01)
                                 updateAspects ();
                         }
-                    } else if (!strncmp (tok + 3, "REF", 3)) {
-                        m_notify->insertMrl (mrl(), URL::decode_string(tok + 7));
+                    } else if (!strncmp (tok + 3, "REF", 3) && m_mrl_notify) {
+                        m_mrl_notify->referenceMrl(URL::decode_string(tok + 7));
                     }
                 } else {
                     char *icy = strstr (tok, "ICY Info");
@@ -821,7 +820,8 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
                             char *e = strchr (t + 12, ';');
                             if (e)
                                 *e = 0;
-                            infoMessage (String (t + 12));
+                            if (m_mrl_notify)
+                                m_mrl_notify->infoMessage (String (t + 12));
                         }
                         handled = true;
                     } else {
@@ -878,6 +878,184 @@ KMPLAYER_NO_EXPORT bool MPlayer::sendCommand (const String & cmd) {
     return false;
 }
 
+MPlayer *GstreamerMPlayerProcess::createGstreamerPlayer (ProcessNotify *notify)
+{
+    MPlayer *gst = new MPlayer (notify, "osso-media-server");
+    gst->setPlayer ("/opt/kmplayer/bin/kgstplayer");
+    gst->setAudioDriver ("pulse");
+    gst->setVideoDriver ("xv");
+    gst->setAutoSubTitles (true);
+    gst->setSupportsPanzoom (true);
+    return gst;
+}
+
+MPlayer *GstreamerMPlayerProcess::createMPlayer (ProcessNotify *notify)
+{
+    MPlayer *mp = new MPlayer (notify, "mplayer");
+    mp->setAudioDriver ("pulse");
+    mp->setVideoDriver ("xv:ck-method=auto");
+    return mp;
+}
+
+GstreamerMPlayerProcess::GstreamerMPlayerProcess (ProcessNotify *ctrl)
+ : Process (ctrl, "gstreamer or mplayer", true),
+   gstreamer (createGstreamerPlayer (this)),
+   mplayer (createMPlayer (this)),
+   process (gstreamer),
+   m_transition_state (NotRunning)
+{}
+
+GstreamerMPlayerProcess::~GstreamerMPlayerProcess ()
+{
+    delete gstreamer;
+    delete mplayer;
+}
+
+bool GstreamerMPlayerProcess::ready (unsigned int xid)
+{
+    process = gstreamer;
+    m_viewer = xid;
+    return process->ready (xid);
+}
+
+bool GstreamerMPlayerProcess::play (const String &url)
+{
+    Process::play (url);
+    if (m_transition_state < Playing) {
+        m_transition_state = Playing;
+        process = gstreamer;
+        process->setMrlBackEndNotify (this);
+        process->setRepeat (m_repeat);
+        process->setHasVideo (!no_video);
+    }
+    return process->play (url);
+}
+
+bool GstreamerMPlayerProcess::stop ()
+{
+    m_transition_state = Ready;
+    return process->stop ();
+}
+
+bool GstreamerMPlayerProcess::quit ()
+{
+    m_transition_state = NotRunning;
+    return process->quit ();
+}
+
+bool GstreamerMPlayerProcess::pause ()
+{
+    return process->pause ();
+}
+
+bool GstreamerMPlayerProcess::seek (int pos, bool absolute)
+{
+    return process->seek (pos, absolute);
+}
+
+//bool GstreamerMPlayerProcess::volume (int pos, bool absolute)
+//{
+    //return process->volume (pos, absolute);
+//}
+
+void GstreamerMPlayerProcess::toggleScale ()
+{
+    return process->toggleScale ();
+}
+
+void GstreamerMPlayerProcess::stateChanged (Process *p,
+        Process::State os, Process::State ns)
+{
+    if (p != process)
+        return;
+
+    if (ns == Process::Playing) {
+        m_transition_state = NotRunning;
+        is_local_file = process->isLocalFile ();
+    } else if (ns == Process::Ready) {
+        if (Playing == m_transition_state &&
+                os > Process::Ready &&
+                process == gstreamer) {
+            process->quit ();
+            process = mplayer;
+            process->ready (m_viewer);
+            process->setMrlBackEndNotify (this);
+            process->setRepeat (m_repeat);
+            process->setHasVideo (!no_video);
+            process->play (m_url);
+            return;
+        }
+    } else if (ns == Process::Buffering) {
+        is_local_file = process->isLocalFile ();
+    }
+    m_state = ns;
+
+    processNotify ()->stateChanged (this, os, ns);
+}
+
+void GstreamerMPlayerProcess::errorMsg (const String & msg)
+{
+    processNotify ()->errorMsg (msg);
+}
+
+void GstreamerMPlayerProcess::setLoading (int perc)
+{
+    processNotify ()->setLoading (perc);
+}
+
+void GstreamerMPlayerProcess::setPosition (int pos)
+{
+    m_position = pos;
+    processNotify ()->setPosition (pos);
+}
+
+void GstreamerMPlayerProcess::setLength (int len)
+{
+    m_length = len;
+    processNotify ()->setLength (len);
+}
+
+void GstreamerMPlayerProcess::setAspect (float aspect)
+{
+    m_aspect = aspect;
+    processNotify ()->setAspect (aspect);
+}
+
+void GstreamerMPlayerProcess::videoDetected ()
+{
+    no_video = false;
+    processNotify ()->videoDetected ();
+}
+
+void GstreamerMPlayerProcess::createDownload (const String &u, const String &t)
+{
+    processNotify ()->createDownload (u, t);
+}
+
+Mrl *GstreamerMPlayerProcess::sourceMrl ()
+{
+    return m_mrl_notify ? m_mrl_notify->sourceMrl () : NULL;
+}
+
+void GstreamerMPlayerProcess::referenceMrl (const String &url)
+{
+    if (m_mrl_notify)
+        m_mrl_notify->referenceMrl (url);
+}
+
+void GstreamerMPlayerProcess::infoMessage (const String &msg)
+{
+    if (m_mrl_notify)
+        m_mrl_notify->infoMessage (msg);
+}
+
+void GstreamerMPlayerProcess::readyToPlay (Process *)
+{
+    if (m_mrl_notify)
+        m_mrl_notify->readyToPlay (this);
+}
+
+
 #ifdef __ARMEL__
 
 KDE_NO_CDTOR_EXPORT NpPlayer::NpPlayer (ProcessNotify * ctrl)
@@ -909,13 +1087,13 @@ static gboolean copyParams (gpointer key, gpointer value, gpointer data) {
     return false; // continue
 }
 
-KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
+KMPLAYER_NO_EXPORT bool NpPlayer::play (const String &uri) {
     if (playing ())
         return pause ();
 
     setState (Buffering);
 
-    Process::play (node);
+    Process::play (uri);
     debugLog () << "NpPlayer::play " << mrl()->mimetype << endl;
 
     if (m_url.isEmpty ()) {
@@ -989,7 +1167,6 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
     }
 
     if (unix_process.start (wd, cmdline)) {
-        m_mrl = node;
         setState (Process::Buffering); //FIXME
         return true;
     }
@@ -999,199 +1176,22 @@ KMPLAYER_NO_EXPORT bool NpPlayer::play (NodePtr node) {
 
 #endif //__ARMEL__
 
-Download::Download (NodePtr &d, ProcessNotify *n, int _fd)
-    : Element (d, id_node_download),
-      job (NULL), notify (n),
-      size (0), bytes (0), fd (_fd)
-{}
-
-Download::~Download ()
-{
-    if (job) {
-        job->kill ();
-        ::close (fd);
-    }
-}
-
-void Download::closed ()
-{
-    file = getAttribute ("file");
-    uri = getAttribute (Ids::attr_src);
-    title = getAttribute (Ids::attr_title);
-    Element::closed ();
-}
-
-void Download::begin ()
-{
-    setState (state_began);
-    bytes = 0;
-    size = 0;
-    debugLog() << "Download " << uri << " as " << file << " cap:" << title << endl;
-    job = asyncGet (this, uri);
-    if (fd > -1) {
-        job->start ();
-    } else {
-        String sz = getAttribute ("size");
-        if (sz.isEmpty ()) {
-            fd = ::open (file, O_WRONLY | O_CREAT | O_TRUNC, 0664);
-        } else {
-            size = strtoll (sz, NULL, 10);
-            fd = ::open (file, O_WRONLY | O_APPEND, 0664);
-            bytes = ::lseek (fd, 0, SEEK_END);
-        }
-        if (fd > 0)
-            job->start (bytes);
-    }
-    notify->updateDownload (this, 0);
-}
-
-void Download::jobData (IOJob *jb, ByteArray &data)
-{
-    int old_perc = size > 0 ? int (100.0 * bytes / size) : 0;
-    if (!bytes) {
-        size = jb->contentLength ();
-        if (size > 0)
-            setAttribute ("size", String::number (size));
-    }
-    bytes += data.size ();
-    int new_perc = size > 0 ? int (100.0 * bytes / size) : 0;
-    if (new_perc >= 100)
-        new_perc = 99;
-    if (old_perc != new_perc)
-        notify->updateDownload (this, new_perc);
-    if (data.size ()) {
-        int nr = write (fd, data.data (), data.size ());
-        if (nr < data.size ()) {
-            String err ("Couldn't write data");
-            if (nr < 0)
-                err += Char (' ') + String (strerror (errno));
-            jb->kill (false);
-            notify->errorMsg (err);
-        }
-    }
-}
-
-void Download::jobResult (IOJob *jb)
-{
-    if (fd > 0)
-        close (fd);
-    fd = -1;
-    job = NULL;
-    if (!jb->error ()) {
-        notify->updateDownload (this, 100);
-        NodePtr safe (this);
-        parentNode ()->removeChild (this); // deletes this
-        notify->updateDownloads ();
-    }
-}
-
-void Download::redirected (IOJob * job, const String & uri)
-{
-}
-
-/*
-String saved_file;
-IOJob *job;
-ProcessNotify *notify;
-unsigned long size;
-unsigned long bytes;
-int fd;*/
-
-Downloader::Downloader (ProcessNotify *ctrl, Application *app)
- : Process (ctrl, "downloader", false),
+Downloader::Downloader (ProcessNotify *pn, Application *app)
+ : Process (pn, "downloader", false),
    application (app) {
 }
 
 Downloader::~Downloader () {
 }
 
-static String genFileName (const String &str, bool skip_on_invalid) {
-    String name;
-    bool valid = true;
-    for (const char *s = (const char *)str; s && *s; ++s)
-        switch (*s) {
-            case '\n':
-            case ';':
-            case ':':
-            case '*':
-            case '?':
-            case ',':
-            case '&':
-            case '>':
-            case '<':
-            case '|':
-            case '"':
-            case '\'':
-            case '/':
-                if (!skip_on_invalid)
-                    return String ();
-                break;
-            default:
-                name += Char (*s);
-        }
-    return name;
-}
-
-void KMPLAYER_NO_EXPORT Downloader::createDownload (const String &url, const String &target_title) {
-    String name, err;
-    String title = target_title;
-    URL uri (url);
-
-    if (uri.isLocalFile ()) {
-        err = "Refuse to download a local file";
-    } else if (uri.protocol () != "http" &&
-            uri.protocol () != "https" &&
-            uri.protocol () != "ftp" &&
-            uri.protocol () != "sftp") {
-        err = String ("Cannot download from protocol ") + uri.protocol ();
-    } else {
-        int pos = url.lastIndexOf (Char ('/'));
-        if (pos > 0)
-            name = genFileName ((const char *) url.mid (pos + 1), false);
-        if (name.isEmpty ())
-            name = title;
-        if (!name.isEmpty ()) {
-            if (title.isEmpty ())
-                title = name;
-            name = genFileName ((const char *) name, true);
-        }
-        String path = String (application->save_folder);
-        if (!name.isEmpty ())
-            path += name;
-        gchar *filename = app_file_chooser (application, (const char *) path,
-                GTK_FILE_CHOOSER_ACTION_SAVE);
-        if (filename) {
-            int fd = open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0664);
-            if (fd > 0) {
-                NodePtr doc = m_notify->downloads ();
-                Element *n = new Download (doc, m_notify, fd);
-                doc->appendChild (n);
-                n->setAttribute (Ids::attr_src, url);
-                n->setAttribute (Ids::attr_title, title);
-                n->setAttribute ("file", filename);
-                n->closed ();
-                n->begin ();
-                processNotify ()->updateDownloads ();
-            } else {
-                err = "Couldn't open file ";
-                err += String (filename) + Char (' ');
-                err += strerror (errno);
-            }
-            app_store_save_directory (application, filename);
-        }
-    }
-    if (!err.isEmpty ())
-        processNotify ()->errorMsg (err);
-}
-
-bool KMPLAYER_NO_EXPORT Downloader::play (NodePtr mrl) {
-    Process::play (mrl);
+bool KMPLAYER_NO_EXPORT Downloader::play (const String &url) {
+    Process::play (url);
     if (!m_url.isEmpty ()) {
         String title;
-        PlaylistRole *title_role = (PlaylistRole*) mrl->role (RolePlaylist);
+        PlaylistRole *title_role = (PlaylistRole*) mrl ()->role (RolePlaylist);
         if (title_role)
             title = title_role->caption ();
-        createDownload (m_url, title);
+        processNotify ()->createDownload (m_url, title);
     }
     setState (NotRunning);
     return false;
